@@ -39,37 +39,6 @@ pub use errors::*;
 mod options;
 pub use options::*;
 
-/// Apache OpenDAL™ Python binding
-///
-/// ## Installation
-///
-/// ```bash
-/// pip install opendal
-/// ```
-///
-/// ## Usage
-///
-/// ```python
-/// import opendal
-///
-/// op = opendal.Operator("fs", root="/tmp")
-/// op.write("test.txt", b"Hello World")
-/// print(op.read("test.txt"))
-/// print(op.stat("test.txt").content_length)
-/// ```
-///
-/// Or using the async API:
-///
-/// ```python
-/// import asyncio
-///
-/// async def main():
-/// op = opendal.AsyncOperator("fs", root="/tmp")
-/// await op.write("test.txt", b"Hello World")
-/// print(await op.read("test.txt"))
-///
-/// asyncio.run(main())
-/// ```
 #[pymodule(gil_used = false)]
 fn _opendal(py: Python, m: &Bound<'_, PyModule>) -> PyResult<()> {
     m.add_class::<Operator>()?;
@@ -91,6 +60,7 @@ fn _opendal(py: Python, m: &Bound<'_, PyModule>) -> PyResult<()> {
     layers_module.add_class::<Layer>()?;
     layers_module.add_class::<RetryLayer>()?;
     layers_module.add_class::<ConcurrentLimitLayer>()?;
+    layers_module.add_class::<MimeGuessLayer>()?;
     m.add_submodule(&layers_module)?;
     py.import("sys")?
         .getattr("modules")?
