@@ -19,44 +19,58 @@ use std::collections::HashMap;
 use std::fmt::Debug;
 use std::fmt::Formatter;
 use std::fmt::Write;
+use std::sync::Arc;
 use std::time::Duration;
 
 use backon::ExponentialBuilder;
 use backon::Retryable;
-use bytes::Bytes;
+use bytes::{Buf, Bytes};
+use http::header::CONTENT_ENCODING;
 use http::header::CONTENT_LENGTH;
-use http::header::CONTENT_RANGE;
 use http::header::CONTENT_TYPE;
 use http::header::HOST;
 use http::header::IF_MATCH;
+use http::header::IF_MODIFIED_SINCE;
 use http::header::IF_NONE_MATCH;
+use http::header::IF_UNMODIFIED_SINCE;
 use http::Request;
 use http::Response;
-use once_cell::sync::Lazy;
 use reqsign::GoogleCredential;
 use reqsign::GoogleCredentialLoader;
 use reqsign::GoogleSigner;
 use reqsign::GoogleToken;
 use reqsign::GoogleTokenLoader;
-use serde::{Deserialize, Serialize};
-use serde_json::json;
+use serde::Deserialize;
+use serde::Serialize;
+use std::sync::LazyLock;
 
 use super::uri::percent_encode_path;
 use crate::raw::*;
 use crate::*;
+use constants::*;
+
+pub mod constants {
+    pub const X_GOOG_ACL: &str = "x-goog-acl";
+    pub const X_GOOG_STORAGE_CLASS: &str = "x-goog-storage-class";
+    pub const X_GOOG_META_PREFIX: &str = "x-goog-meta-";
+}
 
 pub struct GcsCore {
+    pub info: Arc<AccessorInfo>,
     pub endpoint: String,
     pub bucket: String,
     pub root: String,
 
-    pub client: HttpClient,
     pub signer: GoogleSigner,
     pub token_loader: GoogleTokenLoader,
+    pub token: Option<String>,
+    pub scope: String,
     pub credential_loader: GoogleCredentialLoader,
 
     pub predefined_acl: Option<String>,
     pub default_storage_class: Option<String>,
+
+    pub allow_anonymous: bool,
 }
 
 impl Debug for GcsCore {
@@ -69,48 +83,62 @@ impl Debug for GcsCore {
     }
 }
 
-static BACKOFF: Lazy<ExponentialBuilder> =
-    Lazy::new(|| ExponentialBuilder::default().with_jitter());
+static BACKOFF: LazyLock<ExponentialBuilder> =
+    LazyLock::new(|| ExponentialBuilder::default().with_jitter());
 
 impl GcsCore {
-    async fn load_token(&self) -> Result<GoogleToken> {
+    async fn load_token(&self) -> Result<Option<GoogleToken>> {
+        if let Some(token) = &self.token {
+            return Ok(Some(GoogleToken::new(token, usize::MAX, &self.scope)));
+        }
+
         let cred = { || self.token_loader.load() }
-            .retry(&*BACKOFF)
+            .retry(*BACKOFF)
             .await
             .map_err(new_request_credential_error)?;
 
         if let Some(cred) = cred {
-            Ok(cred)
-        } else {
-            Err(Error::new(
-                ErrorKind::ConfigInvalid,
-                "no valid credential found",
-            ))
+            return Ok(Some(cred));
         }
+
+        if self.allow_anonymous {
+            return Ok(None);
+        }
+
+        Err(Error::new(
+            ErrorKind::ConfigInvalid,
+            "no valid credential found",
+        ))
     }
 
-    fn load_credential(&self) -> Result<GoogleCredential> {
+    fn load_credential(&self) -> Result<Option<GoogleCredential>> {
         let cred = self
             .credential_loader
             .load()
             .map_err(new_request_credential_error)?;
 
         if let Some(cred) = cred {
-            Ok(cred)
-        } else {
-            Err(Error::new(
-                ErrorKind::ConfigInvalid,
-                "no valid credential found",
-            ))
+            return Ok(Some(cred));
         }
+
+        if self.allow_anonymous {
+            return Ok(None);
+        }
+
+        Err(Error::new(
+            ErrorKind::ConfigInvalid,
+            "no valid credential found",
+        ))
     }
 
     pub async fn sign<T>(&self, req: &mut Request<T>) -> Result<()> {
-        let cred = self.load_token().await?;
-
-        self.signer
-            .sign(req, &cred)
-            .map_err(new_request_sign_error)?;
+        if let Some(cred) = self.load_token().await? {
+            self.signer
+                .sign(req, &cred)
+                .map_err(new_request_sign_error)?;
+        } else {
+            return Ok(());
+        }
 
         // Always remove host header, let users' client to set it based on HTTP
         // version.
@@ -123,12 +151,14 @@ impl GcsCore {
         Ok(())
     }
 
-    pub async fn sign_query<T>(&self, req: &mut Request<T>, duration: Duration) -> Result<()> {
-        let cred = self.load_credential()?;
-
-        self.signer
-            .sign_query(req, duration, &cred)
-            .map_err(new_request_sign_error)?;
+    pub fn sign_query<T>(&self, req: &mut Request<T>, duration: Duration) -> Result<()> {
+        if let Some(cred) = self.load_credential()? {
+            self.signer
+                .sign_query(req, duration, &cred)
+                .map_err(new_request_sign_error)?;
+        } else {
+            return Ok(());
+        }
 
         // Always remove host header, let users' client to set it based on HTTP
         // version.
@@ -143,7 +173,7 @@ impl GcsCore {
 
     #[inline]
     pub async fn send(&self, req: Request<Buffer>) -> Result<Response<Buffer>> {
-        self.client.send(req).await
+        self.info.http_client().send(req).await
     }
 }
 
@@ -175,6 +205,8 @@ impl GcsCore {
             req = req.header(http::header::RANGE, range.to_header());
         }
 
+        let req = req.extension(Operation::Read);
+
         let req = req.body(Buffer::new()).map_err(new_request_build_error)?;
 
         Ok(req)
@@ -195,6 +227,22 @@ impl GcsCore {
             req = req.header(IF_NONE_MATCH, if_none_match);
         }
 
+        if let Some(if_modified_since) = args.if_modified_since() {
+            req = req.header(
+                IF_MODIFIED_SINCE,
+                format_datetime_into_http_date(if_modified_since),
+            );
+        }
+
+        if let Some(if_unmodified_since) = args.if_unmodified_since() {
+            req = req.header(
+                IF_UNMODIFIED_SINCE,
+                format_datetime_into_http_date(if_unmodified_since),
+            );
+        }
+
+        let req = req.extension(Operation::Read);
+
         let req = req.body(Buffer::new()).map_err(new_request_build_error)?;
 
         Ok(req)
@@ -209,7 +257,7 @@ impl GcsCore {
         let mut req = self.gcs_get_object_request(path, range, args)?;
 
         self.sign(&mut req).await?;
-        self.client.fetch(req).await
+        self.info.http_client().fetch(req).await
     }
 
     pub fn gcs_insert_object_request(
@@ -221,19 +269,19 @@ impl GcsCore {
     ) -> Result<Request<Buffer>> {
         let p = build_abs_path(&self.root, path);
 
-        let mut metadata = HashMap::new();
-        if let Some(storage_class) = &self.default_storage_class {
-            metadata.insert("storageClass", storage_class.as_str());
-        }
-        if let Some(cache_control) = op.cache_control() {
-            metadata.insert("cacheControl", cache_control);
-        }
+        let request_metadata = InsertRequestMetadata {
+            storage_class: self.default_storage_class.as_deref(),
+            cache_control: op.cache_control(),
+            content_type: op.content_type(),
+            content_encoding: op.content_encoding(),
+            metadata: op.user_metadata(),
+        };
 
         let mut url = format!(
             "{}/upload/storage/v1/b/{}/o?uploadType={}&name={}",
             self.endpoint,
             self.bucket,
-            if metadata.is_empty() {
+            if request_metadata.is_empty() {
                 "media"
             } else {
                 "multipart"
@@ -245,42 +293,50 @@ impl GcsCore {
             write!(&mut url, "&predefinedAcl={}", acl).unwrap();
         }
 
+        // Makes the operation conditional on whether the object's current generation
+        // matches the given value. Setting to 0 makes the operation succeed only if
+        // there are no live versions of the object.
+        if op.if_not_exists() {
+            write!(&mut url, "&ifGenerationMatch=0").unwrap();
+        }
+
         let mut req = Request::post(&url);
 
         req = req.header(CONTENT_LENGTH, size.unwrap_or_default());
 
-        if metadata.is_empty() {
-            if let Some(content_type) = op.content_type() {
-                req = req.header(CONTENT_TYPE, content_type);
-            }
-
+        if request_metadata.is_empty() {
+            let req = req.extension(Operation::Write);
+            // If the metadata is empty, we do not set any `Content-Type` header,
+            // since if we had it in the `op.content_type()`, it would be already set in the
+            // `multipart` metadata body and this branch won't be executed.
             let req = req.body(body).map_err(new_request_build_error)?;
             Ok(req)
         } else {
             let mut multipart = Multipart::new();
-
-            multipart = multipart.part(
-                FormDataPart::new("metadata")
-                    .header(
-                        CONTENT_TYPE,
-                        "application/json; charset=UTF-8".parse().unwrap(),
-                    )
-                    .content(json!(metadata).to_string()),
-            );
-
-            let mut media_part = FormDataPart::new("media").content(body);
-
-            if let Some(content_type) = op.content_type() {
-                media_part = media_part.header(
+            let metadata_part = RelatedPart::new()
+                .header(
                     CONTENT_TYPE,
-                    content_type
-                        .parse()
-                        .map_err(|_| Error::new(ErrorKind::Unexpected, "invalid header value"))?,
+                    "application/json; charset=UTF-8".parse().unwrap(),
+                )
+                .content(
+                    serde_json::to_vec(&request_metadata)
+                        .expect("metadata serialization should succeed"),
                 );
-            }
+            multipart = multipart.part(metadata_part);
 
+            // Content-Type must be set, even if it is set in the metadata part
+            let content_type = op
+                .content_type()
+                .unwrap_or("application/octet-stream")
+                .parse()
+                .expect("Failed to parse content-type");
+            let media_part = RelatedPart::new()
+                .header(CONTENT_TYPE, content_type)
+                .content(body);
             multipart = multipart.part(media_part);
-            let req = multipart.apply(Request::post(url))?;
+
+            let req = multipart.apply(Request::post(url).extension(Operation::Write))?;
+
             Ok(req)
         }
     }
@@ -298,18 +354,30 @@ impl GcsCore {
 
         let mut req = Request::put(&url);
 
+        if let Some(user_metadata) = args.user_metadata() {
+            for (key, value) in user_metadata {
+                req = req.header(format!("{X_GOOG_META_PREFIX}{key}"), value)
+            }
+        }
+
         if let Some(content_type) = args.content_type() {
             req = req.header(CONTENT_TYPE, content_type);
         }
 
+        if let Some(content_encoding) = args.content_encoding() {
+            req = req.header(CONTENT_ENCODING, content_encoding);
+        }
+
         if let Some(acl) = &self.predefined_acl {
-            req = req.header("x-goog-acl", acl);
+            req = req.header(X_GOOG_ACL, acl);
         }
 
         if let Some(storage_class) = &self.default_storage_class {
-            req = req.header("x-goog-storage-class", storage_class);
+            req = req.header(X_GOOG_STORAGE_CLASS, storage_class);
         }
 
+        let req = req.extension(Operation::Write);
+
         let req = req.body(body).map_err(new_request_build_error)?;
 
         Ok(req)
@@ -335,6 +403,8 @@ impl GcsCore {
             req = req.header(IF_MATCH, if_match);
         }
 
+        let req = req.extension(Operation::Stat);
+
         let req = req.body(Buffer::new()).map_err(new_request_build_error)?;
 
         Ok(req)
@@ -360,6 +430,8 @@ impl GcsCore {
             req = req.header(IF_MATCH, if_match);
         }
 
+        let req = req.extension(Operation::Stat);
+
         let req = req.body(Buffer::new()).map_err(new_request_build_error)?;
 
         Ok(req)
@@ -412,7 +484,7 @@ impl GcsCore {
             );
         }
 
-        let req = Request::post(uri);
+        let req = Request::post(uri).extension(Operation::Delete);
         let mut req = multipart.apply(req)?;
 
         self.sign(&mut req).await?;
@@ -434,6 +506,7 @@ impl GcsCore {
 
         let mut req = Request::post(req_uri)
             .header(CONTENT_LENGTH, 0)
+            .extension(Operation::Copy)
             .body(Buffer::new())
             .map_err(new_request_build_error)?;
 
@@ -451,22 +524,20 @@ impl GcsCore {
     ) -> Result<Response<Buffer>> {
         let p = build_abs_path(&self.root, path);
 
-        let mut url = format!(
-            "{}/storage/v1/b/{}/o?prefix={}",
-            self.endpoint,
-            self.bucket,
-            percent_encode_path(&p)
-        );
+        let url = format!("{}/storage/v1/b/{}/o", self.endpoint, self.bucket,);
+
+        let mut url = QueryPairsWriter::new(&url);
+        url = url.push("prefix", &percent_encode_path(&p));
+
         if !delimiter.is_empty() {
-            write!(url, "&delimiter={delimiter}").expect("write into string must succeed");
+            url = url.push("delimiter", delimiter);
         }
         if let Some(limit) = limit {
-            write!(url, "&maxResults={limit}").expect("write into string must succeed");
+            url = url.push("maxResults", &limit.to_string());
         }
         if let Some(start_after) = start_after {
             let start_after = build_abs_path(&self.root, &start_after);
-            write!(url, "&startOffset={}", percent_encode_path(&start_after))
-                .expect("write into string must succeed");
+            url = url.push("startOffset", &percent_encode_path(&start_after));
         }
 
         if !page_token.is_empty() {
@@ -476,11 +547,11 @@ impl GcsCore {
             //
             // Don't know how will those tokens be like so this part are copied
             // directly from AWS S3 service.
-            write!(url, "&pageToken={}", percent_encode_path(page_token))
-                .expect("write into string must succeed");
+            url = url.push("pageToken", &percent_encode_path(page_token));
         }
 
-        let mut req = Request::get(&url)
+        let mut req = Request::get(url.finish())
+            .extension(Operation::List)
             .body(Buffer::new())
             .map_err(new_request_build_error)?;
 
@@ -496,22 +567,7 @@ impl GcsCore {
 
         let mut req = Request::post(&url)
             .header(CONTENT_LENGTH, 0)
-            .body(Buffer::new())
-            .map_err(new_request_build_error)?;
-
-        self.sign(&mut req).await?;
-        self.send(req).await
-    }
-
-    pub async fn gcs_initiate_resumable_upload(&self, path: &str) -> Result<Response<Buffer>> {
-        let p = build_abs_path(&self.root, path);
-        let url = format!(
-            "{}/upload/storage/v1/b/{}/o?uploadType=resumable&name={}",
-            self.endpoint, self.bucket, p
-        );
-
-        let mut req = Request::post(&url)
-            .header(CONTENT_LENGTH, 0)
+            .extension(Operation::Write)
             .body(Buffer::new())
             .map_err(new_request_build_error)?;
 
@@ -542,6 +598,8 @@ impl GcsCore {
 
         req = req.header(CONTENT_LENGTH, size);
 
+        let req = req.extension(Operation::Write);
+
         let mut req = req.body(body).map_err(new_request_build_error)?;
 
         self.sign(&mut req).await?;
@@ -567,12 +625,14 @@ impl GcsCore {
         let req = Request::post(&url);
 
         let content = quick_xml::se::to_string(&CompleteMultipartUploadRequest { part: parts })
-            .map_err(new_xml_deserialize_error)?;
+            .map_err(new_xml_serialize_error)?;
         // Make sure content length has been set to avoid post with chunked encoding.
         let req = req.header(CONTENT_LENGTH, content.len());
         // Set content-type to `application/xml` to avoid mixed with form post.
         let req = req.header(CONTENT_TYPE, "application/xml");
 
+        let req = req.extension(Operation::Write);
+
         let mut req = req
             .body(Buffer::from(Bytes::from(content)))
             .map_err(new_request_build_error)?;
@@ -597,71 +657,83 @@ impl GcsCore {
         );
 
         let mut req = Request::delete(&url)
+            .extension(Operation::Write)
             .body(Buffer::new())
             .map_err(new_request_build_error)?;
         self.sign(&mut req).await?;
         self.send(req).await
     }
 
-    pub fn gcs_upload_in_resumable_upload(
-        &self,
-        location: &str,
-        size: u64,
-        written: u64,
-        body: Buffer,
-    ) -> Result<Request<Buffer>> {
-        let mut req = Request::put(location);
+    pub fn build_metadata_from_object_response(path: &str, data: Buffer) -> Result<Metadata> {
+        let meta: GetObjectJsonResponse =
+            serde_json::from_reader(data.reader()).map_err(new_json_deserialize_error)?;
 
-        let range_header = format!("bytes {}-{}/*", written, written + size - 1);
+        let mut m = Metadata::new(EntryMode::from_path(path));
 
-        req = req
-            .header(CONTENT_LENGTH, size)
-            .header(CONTENT_RANGE, range_header);
+        m.set_etag(&meta.etag);
+        m.set_content_md5(&meta.md5_hash);
 
-        // Set body
-        let req = req.body(body).map_err(new_request_build_error)?;
+        let size = meta
+            .size
+            .parse::<u64>()
+            .map_err(|e| Error::new(ErrorKind::Unexpected, "parse u64").set_source(e))?;
+        m.set_content_length(size);
+        if !meta.content_type.is_empty() {
+            m.set_content_type(&meta.content_type);
+        }
 
-        Ok(req)
-    }
+        if !meta.content_encoding.is_empty() {
+            m.set_content_encoding(&meta.content_encoding);
+        }
 
-    pub async fn gcs_complete_resumable_upload(
-        &self,
-        location: &str,
-        written: u64,
-        size: u64,
-        body: Buffer,
-    ) -> Result<Response<Buffer>> {
-        let mut req = Request::post(location)
-            .header(CONTENT_LENGTH, size)
-            .header(
-                CONTENT_RANGE,
-                format!(
-                    "bytes {}-{}/{}",
-                    written,
-                    written + size - 1,
-                    written + size
-                ),
-            )
-            .body(body)
-            .map_err(new_request_build_error)?;
+        if !meta.cache_control.is_empty() {
+            m.set_cache_control(&meta.cache_control);
+        }
 
-        self.sign(&mut req).await?;
+        if !meta.content_disposition.is_empty() {
+            m.set_content_disposition(&meta.content_disposition);
+        }
 
-        self.send(req).await
-    }
+        if !meta.generation.is_empty() {
+            m.set_version(&meta.generation);
+        }
 
-    pub async fn gcs_abort_resumable_upload(&self, location: &str) -> Result<Response<Buffer>> {
-        let mut req = Request::delete(location)
-            .header(CONTENT_LENGTH, 0)
-            .body(Buffer::new())
-            .map_err(new_request_build_error)?;
+        m.set_last_modified(parse_datetime_from_rfc3339(&meta.updated)?);
 
-        self.sign(&mut req).await?;
+        if !meta.metadata.is_empty() {
+            m.with_user_metadata(meta.metadata);
+        }
 
-        self.send(req).await
+        Ok(m)
     }
 }
 
+#[derive(Debug, Serialize)]
+#[serde(default, rename_all = "camelCase")]
+pub struct InsertRequestMetadata<'a> {
+    #[serde(skip_serializing_if = "Option::is_none")]
+    content_type: Option<&'a str>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    content_encoding: Option<&'a str>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    storage_class: Option<&'a str>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    cache_control: Option<&'a str>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    metadata: Option<&'a HashMap<String, String>>,
+}
+
+impl InsertRequestMetadata<'_> {
+    pub fn is_empty(&self) -> bool {
+        self.content_type.is_none()
+            && self.content_encoding.is_none()
+            && self.storage_class.is_none()
+            && self.cache_control.is_none()
+            // We could also put content-encoding in the url parameters
+            && self.content_encoding.is_none()
+            && self.metadata.is_none()
+    }
+}
 /// Response JSON from GCS list objects API.
 ///
 /// refer to https://cloud.google.com/storage/docs/json_api/v1/objects/list for details
@@ -714,10 +786,101 @@ pub struct CompleteMultipartUploadRequestPart {
     pub etag: String,
 }
 
+/// The raw json response returned by [`get`](https://cloud.google.com/storage/docs/json_api/v1/objects/get)
+#[derive(Debug, Default, Deserialize)]
+#[serde(default, rename_all = "camelCase")]
+struct GetObjectJsonResponse {
+    /// GCS will return size in string.
+    ///
+    /// For example: `"size": "56535"`
+    size: String,
+    /// etag is not quoted.
+    ///
+    /// For example: `"etag": "CKWasoTgyPkCEAE="`
+    etag: String,
+    /// RFC3339 styled datetime string.
+    ///
+    /// For example: `"updated": "2022-08-15T11:33:34.866Z"`
+    updated: String,
+    /// Content md5 hash
+    ///
+    /// For example: `"md5Hash": "fHcEH1vPwA6eTPqxuasXcg=="`
+    md5_hash: String,
+    /// Content type of this object.
+    ///
+    /// For example: `"contentType": "image/png",`
+    content_type: String,
+    /// Content encoding of this object
+    ///
+    /// For example: "contentEncoding": "br"
+    content_encoding: String,
+    /// Content disposition of this object
+    content_disposition: String,
+    /// Cache-Control directive for the object data.
+    cache_control: String,
+    /// Content generation of this object. Used for object versioning and soft delete.
+    generation: String,
+    /// Custom metadata of this object.
+    ///
+    /// For example: `"metadata" : { "my-key": "my-value" }`
+    metadata: HashMap<String, String>,
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
 
+    #[test]
+    fn test_deserialize_get_object_json_response() {
+        let content = r#"{
+    "kind": "storage#object",
+    "id": "example/1.png/1660563214863653",
+    "selfLink": "https://www.googleapis.com/storage/v1/b/example/o/1.png",
+    "mediaLink": "https://content-storage.googleapis.com/download/storage/v1/b/example/o/1.png?generation=1660563214863653&alt=media",
+    "name": "1.png",
+    "bucket": "example",
+    "generation": "1660563214863653",
+    "metageneration": "1",
+    "contentType": "image/png",
+    "contentEncoding": "br",
+    "contentDisposition": "attachment",
+    "cacheControl": "public, max-age=3600",
+    "storageClass": "STANDARD",
+    "size": "56535",
+    "md5Hash": "fHcEH1vPwA6eTPqxuasXcg==",
+    "crc32c": "j/un9g==",
+    "etag": "CKWasoTgyPkCEAE=",
+    "timeCreated": "2022-08-15T11:33:34.866Z",
+    "updated": "2022-08-15T11:33:34.866Z",
+    "timeStorageClassUpdated": "2022-08-15T11:33:34.866Z",
+    "metadata" : {
+        "location" : "everywhere"
+  }
+}"#;
+
+        let meta = GcsCore::build_metadata_from_object_response("1.png", content.into())
+            .expect("parse metadata should not fail");
+
+        assert_eq!(meta.content_length(), 56535);
+        assert_eq!(
+            meta.last_modified(),
+            Some(
+                parse_datetime_from_rfc3339("2022-08-15T11:33:34.866Z")
+                    .expect("parse date should not fail")
+            )
+        );
+        assert_eq!(meta.content_md5(), Some("fHcEH1vPwA6eTPqxuasXcg=="));
+        assert_eq!(meta.etag(), Some("CKWasoTgyPkCEAE="));
+        assert_eq!(meta.content_type(), Some("image/png"));
+        assert_eq!(meta.content_encoding(), Some("br"));
+        assert_eq!(meta.content_disposition(), Some("attachment"));
+        assert_eq!(meta.cache_control(), Some("public, max-age=3600"));
+        assert_eq!(meta.version(), Some("1660563214863653"));
+
+        let metadata = HashMap::from_iter([("location".to_string(), "everywhere".to_string())]);
+        assert_eq!(meta.user_metadata(), Some(&metadata));
+    }
+
     #[test]
     fn test_deserialize_list_response() {
         let content = r#"
