@@ -21,58 +21,18 @@ use std::sync::Arc;
 
 use http::Response;
 use http::StatusCode;
-use serde::Deserialize;
-use serde::Serialize;
 use tokio::sync::Mutex;
 
 use super::core::*;
 use crate::raw::*;
+use crate::services::IcloudConfig;
 use crate::*;
 
-/// Config for icloud services support.
-#[derive(Default, Serialize, Deserialize, Clone, PartialEq, Eq)]
-#[serde(default)]
-#[non_exhaustive]
-pub struct IcloudConfig {
-    /// root of this backend.
-    ///
-    /// All operations will happen under this root.
-    ///
-    /// default to `/` if not set.
-    pub root: Option<String>,
-    /// apple_id of this backend.
-    ///
-    /// apple_id must be full, mostly like `example@gmail.com`.
-    pub apple_id: Option<String>,
-    /// password of this backend.
-    ///
-    /// password must be full.
-    pub password: Option<String>,
-
-    /// Session
-    ///
-    /// token must be valid.
-    pub trust_token: Option<String>,
-    /// ds_web_auth_token must be set in Session
-    pub ds_web_auth_token: Option<String>,
-    /// enable the china origin
-    /// China region `origin` Header needs to be set to "https://www.icloud.com.cn".
-    ///
-    /// otherwise Apple server will return 302.
-    pub is_china_mainland: bool,
-}
-
-impl Debug for IcloudConfig {
-    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
-        let mut d = f.debug_struct("IcloudBuilder");
-        d.field("root", &self.root);
-        d.field("is_china_mainland", &self.is_china_mainland);
-        d.finish_non_exhaustive()
-    }
-}
-
 impl Configurator for IcloudConfig {
-    fn into_builder(self) -> impl Builder {
+    type Builder = IcloudBuilder;
+
+    #[allow(deprecated)]
+    fn into_builder(self) -> Self::Builder {
         IcloudBuilder {
             config: self,
             http_client: None,
@@ -92,6 +52,7 @@ pub struct IcloudBuilder {
     ///
     /// This API is part of OpenDAL's Raw API. `HttpClient` could be changed
     /// during minor updates.
+    #[deprecated(since = "0.53.0", note = "Use `Operator::update_http_client` instead")]
     pub http_client: Option<HttpClient>,
 }
 
@@ -185,6 +146,8 @@ impl IcloudBuilder {
     ///
     /// This API is part of OpenDAL's Raw API. `HttpClient` could be changed
     /// during minor updates.
+    #[deprecated(since = "0.53.0", note = "Use `Operator::update_http_client` instead")]
+    #[allow(deprecated)]
     pub fn http_client(mut self, client: HttpClient) -> Self {
         self.http_client = Some(client);
         self
@@ -228,19 +191,32 @@ impl Builder for IcloudBuilder {
                 .with_context("service", Scheme::Icloud)),
         }?;
 
-        let client = if let Some(client) = self.http_client {
-            client
-        } else {
-            HttpClient::new().map_err(|err| {
-                err.with_operation("Builder::build")
-                    .with_context("service", Scheme::Icloud)
-            })?
-        };
-
         let session_data = SessionData::new();
 
+        let info = AccessorInfo::default();
+        info.set_scheme(Scheme::Icloud)
+            .set_root(&root)
+            .set_native_capability(Capability {
+                stat: true,
+                stat_has_content_length: true,
+                stat_has_last_modified: true,
+
+                read: true,
+
+                shared: true,
+                ..Default::default()
+            });
+
+        // allow deprecated api here for compatibility
+        #[allow(deprecated)]
+        if let Some(client) = self.http_client {
+            info.update_http_client(|_| client);
+        }
+
+        let accessor_info = Arc::new(info);
+
         let signer = IcloudSigner {
-            client: client.clone(),
+            info: accessor_info.clone(),
             data: session_data,
             apple_id,
             password,
@@ -253,6 +229,7 @@ impl Builder for IcloudBuilder {
         let signer = Arc::new(Mutex::new(signer));
         Ok(IcloudBackend {
             core: Arc::new(IcloudCore {
+                info: accessor_info,
                 signer: signer.clone(),
                 root,
                 path_cache: PathCacher::new(IcloudPathQuery::new(signer.clone())),
@@ -268,22 +245,16 @@ pub struct IcloudBackend {
 
 impl Access for IcloudBackend {
     type Reader = HttpBody;
-    type BlockingReader = ();
     type Writer = ();
-    type BlockingWriter = ();
     type Lister = ();
+    type Deleter = ();
+    type BlockingReader = ();
+    type BlockingWriter = ();
     type BlockingLister = ();
+    type BlockingDeleter = ();
 
     fn info(&self) -> Arc<AccessorInfo> {
-        let mut ma = AccessorInfo::default();
-        ma.set_scheme(Scheme::Icloud)
-            .set_root(&self.core.root)
-            .set_native_capability(Capability {
-                stat: true,
-                read: true,
-                ..Default::default()
-            });
-        ma.into()
+        self.core.info.clone()
     }
 
     async fn stat(&self, path: &str, _: OpStat) -> Result<RpStat> {
@@ -320,7 +291,7 @@ impl Access for IcloudBackend {
             _ => {
                 let (part, mut body) = resp.into_parts();
                 let buf = body.to_buffer().await?;
-                Err(parse_error(Response::from_parts(part, buf)).await?)
+                Err(parse_error(Response::from_parts(part, buf)))
             }
         }
     }
